﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter {
  using System;
  using HeuristicLab.Core;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Data.Pool;
  using HeuristicLab.Problems.ProgramSynthesis.Push.Expressions;
  using HeuristicLab.Random;

  public class PushInterpreterPool {
    private readonly ObjectPool<PooledPushInterpreter> pool;
    private ManagedPoolProvider<PushProgram> pushProgramPoolProvider;

    public PushInterpreterPool(PushConfiguration config = null)
      : this(Environment.ProcessorCount * 2, 1024, null, config) {
    }

    public PushInterpreterPool(int size, int pushProgramPoolPartitionSize, int? maxPartitionCount = null, PushConfiguration config = null) {
      PushGpConfiguration = config ?? new PushConfiguration();

      pushProgramPoolProvider = new ManagedPoolProvider<PushProgram>(pushProgramPoolPartitionSize, maxPartitionCount);
      pushProgramPoolProvider.InitDummyPartition(() => new PushProgram());

      pool = new ObjectPool<PooledPushInterpreter>(() => new PooledPushInterpreter(this, PushGpConfiguration, pushProgramPoolProvider), size);
    }

    public PushConfiguration PushGpConfiguration { get; private set; }

    public PooledPushInterpreter GetInstance(IRandom random = null) {
      var interpreter = this.pool.Allocate();
      interpreter.Random = random ?? new FastRandom();

      return interpreter;
    }

    public void Free(PooledPushInterpreter interpreter) {
      this.pool.Free(interpreter);
    }
  }
}
